package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.Solver;
import dk.brics.tajs.analysis.SpecialVars;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.flowgraph.SourceLocation;

import java.util.Set;

import static dk.brics.tajs.util.Collections.newSet;

public class AllParamSensitive implements ContextSensitivityInferer {
    private final Set<SourceLocation> doesNotNeedToBeContextSensitive;

    public AllParamSensitive(Set<ContextSensitivityInferenceManager.FunctionLocation> doesNotNeedToBeContextSensitive) {
        this.doesNotNeedToBeContextSensitive = newSet();
        for (ContextSensitivityInferenceManager.FunctionLocation function : doesNotNeedToBeContextSensitive) {
            this.doesNotNeedToBeContextSensitive.add(function.getSourceLocation());
        }

    }

    @Override
    public void inferLocals(Function f, Solver.SolverInterface c) {
        if (doesNotNeedToBeContextSensitive.contains(f.getSourceLocation())) {
            return;
        }
        SpecialVars specialVars = c.getAnalysis().getSpecialVars();
        for (String parameterName : f.getParameterNames()) {
            specialVars.addToSpecialVars(f, parameterName);
        }
    }

}
