package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.ContextArguments;
import dk.brics.tajs.analysis.FunctionCalls;
import dk.brics.tajs.analysis.Solver;
import dk.brics.tajs.analysis.State;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.flowgraph.SourceLocation;
import dk.brics.tajs.solver.GenericSolver;
import dk.brics.tajs.util.Collections;

public class ContextSensitivityInferenceManager {
    private static ContextSensitivityInferenceManager instance;
    private final ContextSensitivityInferer inferer;
    private final CallEntryContextCreator creator;

    private ContextSensitivityInferenceManager() {
        //inferer = new CompositeCSI(new AllIVarsSensitive(doesNotNeedToBeContextSensitive), new AllParamSensitive(doesNotNeedToBeContextSensitive));
        inferer = new AllIVarsSensitive(Collections.<FunctionLocation>newSet());
        //creator = new AllParamContextCreator();
        creator = new PreciseArgumentsContextCreator();
    }

    private ContextSensitivityInferenceManager(ContextSensitivityInferer inferer, CallEntryContextCreator creator) {
        this.inferer = inferer;
        this.creator = creator;
    }

    public static void inferLocals(Function f, Solver.SolverInterface c) {
        instance.inferer.inferLocals(f, c); // TODO avoid redundant inferences..
    }

    public static void init() { // default
        instance = new ContextSensitivityInferenceManager();
    }

    public static void init(ContextSensitivityInferer inferer, CallEntryContextCreator creator) {
        instance = new ContextSensitivityInferenceManager(inferer, creator);
    }

    public static ContextArguments getContextArguments(Function function, FunctionCalls.CallInfo call, State state, GenericSolver.SolverInterface c) {
        return instance.creator.getContextArguments(function, call, state, c);
    }

    public static class FunctionLocation {

        private final SourceLocation sourceLocation;

        public SourceLocation getSourceLocation() {
            return sourceLocation;
        }

        public FunctionLocation(SourceLocation sourceLocation) {
            this.sourceLocation = sourceLocation;
        }
    }
}
