package dk.brics.tajs.csi;

import dk.brics.tajs.flowgraph.AbstractNode;
import dk.brics.tajs.flowgraph.BasicBlock;
import dk.brics.tajs.flowgraph.Function;

import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import static dk.brics.tajs.util.Collections.newSet;

public class FlowGraphUtil {
    public static void begin(Function function, Map<Function, Integer> counter) {
        if (!counter.containsKey(function)) {
            counter.put(function, 0);
        }
        counter.put(function, counter.get(function) + 1);
    }

    public static int get(Function function, Map<Function, Integer> counter) {
        return counter.containsKey(function) ? counter.get(function) : 0;
    }

    public static void end(Function function, Map<Function, Integer> counter) {
        counter.put(function, counter.get(function) - 1);
    }

    /**
     * Finds the nodes of a specific type in a block and its successors, does <emph>not</emph> cross function-boundaries
     */
    public static <T> Set<T> extractNodes(BasicBlock block, Class<T> nodeType) {
        final Function function = block.getFunction();
        Set<BasicBlock> seen = newSet();
        LinkedList<BasicBlock> workList = new LinkedList<>();
        workList.add(block);
        Set<T> nodes = newSet();
        while (!workList.isEmpty()) {
            BasicBlock current = workList.pop();
            if (seen.contains(current)) {
                continue;
            }
            seen.add(current);
            workList.addAll(current.getSuccessors());
            for (AbstractNode node : current.getNodes()) {
                if (nodeType.isAssignableFrom(node.getClass())) {
                    nodes.add((T) node);
                }
            }
        }
        return nodes;
    }
}
