package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.ContextArguments;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.flowgraph.jsnodes.CallNode;
import dk.brics.tajs.flowgraph.jsnodes.NewObjectNode;
import dk.brics.tajs.flowgraph.jsnodes.ReadVariableNode;

public class LiteralContextSensitivityManager {

    private static LiteralContextSensitivityManager instance;
    private final LiteralContextSensitivityStrategy strategy;

    public LiteralContextSensitivityManager(LiteralContextSensitivityStrategy strategy) {
        this.strategy = strategy;
    }

    public static void init(LiteralContextSensitivityStrategy strategy) {
        instance = new LiteralContextSensitivityManager(strategy);
    }

    public static boolean shoudArrayBeSensitive(CallNode node, ContextArguments arguments) {
        return instance.strategy.shoudArrayBeSensitive(node, arguments);
    }

    public static boolean shoudObjectBeSensitive(NewObjectNode node, ContextArguments arguments) {
        return instance.strategy.shoudObjectBeSensitive(node, arguments);
    }

    public static void registerEndForIn(Function function) {
        instance.strategy.registerEndForIn(function);
    }

    public static void registerBeginForIn(Function function) {
        instance.strategy.registerBeginForIn(function);

    }

    public static void registerBeginObjectLiteral(NewObjectNode node) {
        instance.strategy.registerBeginObjectLiteral(node);
    }

    public static void registerEndObjectLiteral(NewObjectNode node) {
        instance.strategy.registerEndObjectLiteral(node);
    }

    public static void registerVariableRead(Function function, ReadVariableNode readVariableNode) {
        instance.strategy.registerVariableRead(function, readVariableNode);
    }
}
