package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.Solver;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.solver.Message;

import java.util.Set;

import static dk.brics.tajs.util.Collections.newSet;

public class LoopVariableSensitive implements ContextSensitivityInferer {
    private SyntaxBasedLoopVariableIdentifierManager.IdentificationStrategy identifier;

    public LoopVariableSensitive(SyntaxBasedLoopVariableIdentifierManager.IdentificationStrategy identifier) {
        this.identifier = identifier;
    }

    @Override
    public void inferLocals(Function f, Solver.SolverInterface c) {
        Set<String> loopVariables = newSet();
        for (String variable : f.getVariableNames()) {
            if (identifier.isLoopVariable(f, variable)) {
                loopVariables.add(variable);
            }
        }
        if (!loopVariables.isEmpty()) {
            c.getMonitoring().addMessage(f.getEntry().getFirstNode(), Message.Severity.TAJS_META, "Loop vars: " + loopVariables);
            for (String loopVariable : loopVariables) {
                c.getAnalysis().getSpecialVars().addToSpecialVars(f, loopVariable);
            }
        }
    }
}
