package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.ContextArguments;
import dk.brics.tajs.analysis.FunctionCalls;
import dk.brics.tajs.analysis.State;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.lattice.UnknownValueResolver;
import dk.brics.tajs.lattice.Value;
import dk.brics.tajs.solver.GenericSolver;

import java.util.List;

import static dk.brics.tajs.util.Collections.newList;

public class PreciseArgumentsContextCreator implements CallEntryContextCreator {
    private final ValuePredicate precisionPredicateSmall = new PreciseValuePredicate(1);

    @Override
    public ContextArguments getContextArguments(Function function, FunctionCalls.CallInfo call, State state, GenericSolver.SolverInterface c) {
        if (call.isUnknownNumberOfArgs()) {
            return new ContextArguments(call.getUnknownArg());
        }
        List<Value> arguments = newList();
        // does not add extra nulls for imprecis arguments: that would distinguish contexts based on the number of arguments
        int maxPreciseArgument = -1;
        for (int i = 0; i < call.getNumberOfArgs(); i++) {
            Value argument = UnknownValueResolver.getRealValue(call.getArg(i), state);
            boolean isPrecise = precisionPredicateSmall.apply(argument, c);
            if (isPrecise) {
                arguments.add(argument);
                maxPreciseArgument = i;
            } else {
                arguments.add(null);
            }
        }
        arguments = newList(arguments.subList(0, maxPreciseArgument + 1));

        List<String> parameterNames = function.getParameterNames();
        return new ContextArguments(parameterNames, arguments);
    }
}
