package dk.brics.tajs.csi;

import dk.brics.tajs.analysis.nativeobjects.concrete.Gamma;
import dk.brics.tajs.lattice.ObjectLabel;
import dk.brics.tajs.lattice.Value;
import dk.brics.tajs.solver.GenericSolver;

public class PreciseValuePredicate implements ValuePredicate {

    private final int objectLimit;

    public PreciseValuePredicate(int objectLimit) {
        this.objectLimit = objectLimit;
    }

    @Override
    public boolean apply(Value value, GenericSolver.SolverInterface c) {
        if (value.typeSize() == 1) {
            if (Gamma.isConcreteString(value, c)) {
                return true;
            }
            if (value.isMaybeObject()) {
                for (ObjectLabel objectLabel : value.getObjectLabels()) {
                    switch (objectLabel.getKind()) {
                        case OBJECT:
                            return value.getObjectSourceLocations().size() <= objectLimit;
                        case FUNCTION:
                        case ARGUMENTS:
                        case ARRAY:
                            return value.getObjectLabels().size() == 1;
                    }
                }
            }
        }
        return false;
    }
}
