/*
 * Copyright 2009-2013 Aarhus University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dk.brics.tajs.solver;

import java.util.Collection;
import java.util.List;

import dk.brics.tajs.Main;
import dk.brics.tajs.flowgraph.AbstractNode;
import dk.brics.tajs.flowgraph.BasicBlock;
import dk.brics.tajs.flowgraph.FlowGraph;
import dk.brics.tajs.flowgraph.Function;
import dk.brics.tajs.flowgraph.jsnodes.ReadPropertyNode;
import dk.brics.tajs.lattice.Value;

/**
 * Records various information during fixpoint solving.
 */
public interface IMonitoring<BlockStateType extends IBlockState<BlockStateType, ?, ?>, ContextType extends IContext<ContextType>> {

	/**
	 * Registers a node transfer occurrence.
	 */
	void visitNodeTransfer(AbstractNode n);

	/**
	 * Registers a block transfer occurrence.
	 */
	void visitBlockTransfer(BasicBlock b, ContextType context);

	/**
	 * Registers new dataflow being propagated.
	 */
	void visitNewFlow(BasicBlock b, IContext<?> c, IBlockState<?, ?, ?> s, String diff, String info);

    /**
     * Registers the *end* of a block tranfer, and the resulting state
     */
    void visitPostBlockTransfer(BasicBlock block, BlockStateType state);

    /**
	 * Registers a recovery of an unknown value.
	 */
	void visitUnknownValueResolve(boolean partial, boolean scanning);

	/**
	 * Registers a property recovery graph size.
	 */
	void visitRecoveryGraph(int size);

	/**
	 * Registers the given function in the scan phase. (Invoked once on each function.)
	 */
	void visitFunction(Function f, Collection<BlockStateType> entry_states);

	/**
	 * Registers a maybe reachable node.
	 */
	void visitReachableNode(AbstractNode n);

	/**
	 * Registers a state join operation.
	 */
	void visitJoin();

 	/**
	 * Initializes messages. Messages are not generated until this method is called.
	 */
	void beginScanPhase(FlowGraph fg);

 	/**
	 * Finalizes messages.
	 */
	void endScanPhase(FlowGraph fg);

    /**
     * Registers a visit of a dynamic property access
     */
    void visitNonFixedPropertyRead(ReadPropertyNode n, Value propertyval);

    void beginPhase(Main.Phase phase);

    void endPhase(Main.Phase phase);

}
